==> stam_1.4.0.tar.gz: stam/R/03_net.R <==
################################################################################
#
#   03_stamNet.R
#
#   Structural information on a whole graph given a specific
#   affymetrix chip and a root node. Holds an array of GO-nodes
#   in postorder, introducing additional nodes for gene annotations.
#   Nodes holding only one child after pruning all children
#   without gene annotations are replaced by that child.
#
#   The implementation relies on Bioconductor data packages to
#   extract chip-specific data as well as the GO structure.
#
#   written by Claudio Lottaz (COMPDIAG-MPI-MG), in August 2004
#
################################################################################
#
#   Class Definition and Constructor
#

setClass("stamNet", 
         representation(chip     = "character",     # chip to produce data
                        root     = "character",     # where to start the analyses
                        chippkg  = "character",     # package with chip-related data
                        GOpkg    = "character",     # package with GO-related data
                        nodes    = "list",          # stamINodes and stamLeafs
                        leafs    = "numeric",       # indeces to leafs
                        inodes   = "numeric",       # indeces to inner nodes
                        probes   = "environment"))  # holds indeces of affyids


setMethod("initialize", "stamNet", 
  function(.Object, chip="hgu95av2", root="GO:0008150", probes=character(0)) {

    .Object@chip <- chip
    .Object@root <- root

    # find chip-specific data
    if (sum(.packages(all=TRUE) == chip) == 0) {
      require(reposTools)
      install.packages2(chip, type="Source")
      if (sum(.packages(all=TRUE) == chip) == 0) 
        stop("No data for chip ", chip, " has been found")
    }
    chipdsc <- packageDescription(chip)
    GOdsc   <- packageDescription("GO")
    .Object@chippkg <- paste("Chip data package:\n  version", chipdsc$Version, 
                             "\n  ", chipdsc$Packaged)
    .Object@GOpkg   <- paste("GO package:\n  version",   GOdsc$Version,   
                             "\n  ", GOdsc$Packaged)
    require(package=chip, character.only=TRUE)

    # check probes
    chipprobes <- getprobes(chip)
    if (length(probes) > 0) { 
      .Object@probes <- new.env(hash=TRUE)
      for (i in 1:length(probes)) {
        if (!exists(probes[i], chipprobes)) 
          stop("Rownames of expression matrix do not match '", chip, "'")
        assign(probes[i], i, .Object@probes)
      }
    } else { .Object@probes <- chipprobes }

    # construct the graph
    chipenv <- get(paste(chip, "GO2PROBE", sep=""), paste("package:", chip, sep=""))
    .stam.nodes     <<- list()
    .stam.nnodes    <<- 0
    .stam.leafs     <<- logical(0)    
    .stam.GOidx     <<- new.env(hash=TRUE)
    environment(stam.addNode) <- environment()
    postorder.traverse(root, stam.addNode)
    names(.stam.nodes) <- sapply(.stam.nodes, function(x) x@ID)
    .Object@nodes <- .stam.nodes
    .Object@leafs  <- which(.stam.leafs)
    .Object@inodes <- which(!.stam.leafs)
    rm(.stam.leafs, .stam.nodes, .stam.nnodes, .stam.GOidx, envir=.GlobalEnv)
    return(.Object)
  }
)


################################################################################
#
#   Postorder Traversal
#


postorder.traverse <- function(ID, action, env) { 
  postorder.traverse.rec <- function(ID, action, env) {
    if (exists(ID, env)) return(FALSE)
    assign(ID, 1, env)
    nvisited <- get("nvisited", env)
    assign("nvisited", nvisited + 1, env)
    if ((nvisited %% 500) == 0)
      cat("registered",.stam.nnodes,"of",nvisited,"visited nodes\n")

    for (c in getGOchildren(ID)) postorder.traverse.rec(c, action, env)
    return(action(ID))
  }

  env <- new.env(hash=TRUE)
  assign("nvisited", 0, env)
  assign("visited", character(0), env)
  postorder.traverse.rec(ID, action, env)
  cat("registered", .stam.nnodes, "of", get("nvisited", env), "visited nodes\n")
  rm(env)
}

stam.addNode <- function(ID) {
  children <- stam.getidx(getGOchildren(ID), .stam.GOidx)
  nchildren <- length(children)
  if (exists(ID, chipenv)) {  # current node has probesets annotated
    l <- new("stamLeaf",  ID=ID, chip=chip, .Object@probes)
    if (nchildren > 0) { 
      chnames <- c(unlist(lapply(children, function(c) .stam.nodes[[c]]@ID)), ID)
      children <- c(children, .stam.nnodes+1)
      names(children) <- chnames
      n <- new("stamINode", ID=ID, children=children)
      .stam.nnodes <<- .stam.nnodes + 2
      l@supNode <- TRUE
      .stam.nodes[[.stam.nnodes-1]] <<- l; .stam.leafs[.stam.nnodes-1] <<- TRUE
      .stam.nodes[[.stam.nnodes]]   <<- n; .stam.leafs[.stam.nnodes]   <<- FALSE
      assign(ID, .stam.nnodes, .stam.GOidx)
    } else {
      .stam.nnodes <<- .stam.nnodes + 1
      .stam.nodes[[.stam.nnodes]] <<- l; .stam.leafs[.stam.nnodes] <<- TRUE
      assign(ID, .stam.nnodes, .stam.GOidx)
    }
  } else { # current node has no probesets annotated
    if (nchildren > 1) { 
      names(children) <- unlist(lapply(children, function(c) .stam.nodes[[c]]@ID))
      n <- new("stamINode", ID=ID, children=children)
      .stam.nnodes <<- .stam.nnodes + 1
      .stam.nodes[[.stam.nnodes]] <<- n; .stam.leafs[.stam.nnodes] <<- FALSE
      assign(ID, .stam.nnodes, .stam.GOidx)
    } else {
      if (nchildren == 1) { # replace current node by its only childee
        child <- children[1]
	        reps <- length(.stam.nodes[[child]]@replacedParents)
        .stam.nodes[[child]]@replacedParents[reps+1] <<- ID
        assign(ID, child, .stam.GOidx)
      } else {
        return(FALSE)
      }
    }
  }
  return(TRUE)
}

################################################################################
#
#   Reports
#

setMethod("print", "stamNet", 
  function(x, ...) {
    allGOenv <- get(paste(x@chip, "GO2ALLPROBES", sep=""),
                    paste("package:", x@chip, sep=""))
    nprobes <- length(unique(get(x@root, allGOenv)))
    allprobes <- length(ls(x@probes))

    cat("stamNet on chip ", x@chip, "\n", x@chippkg, "\n",
        "Gene Ontology ", x@GOpkg, ")\n",
        "root is set to \n", x@root, " (", getGOterm(x@root), ")\n", 
        "holds ", length(x@nodes), " nodes, ", length(x@leafs), 
        " of which are leafs and ", length(x@inodes), " are inner nodes\n", 
        nprobes, " of ", allprobes, " probesets have annotations", 
        " (", round(nprobes/allprobes*100, 1), "%)\n", sep="")

  }
)

setMethod("writeHTML", "stamNet", 
  function(x, nonodes=FALSE, ...) {
    allGOenv <- get(paste(x@chip, "GO2ALLPROBES", sep=""),
                    paste("package:", x@chip, sep=""))

    nprobes <- length(unique(get(x@root, allGOenv)))
    chippkg <- sub(" / ", "<BR>", x@chippkg)
    GOpkg <- sub(" / ", "<BR>", x@GOpkg)

    catHTML(HR(), NAME("netinfo"), H2("Annotation Information"), 
            P(TABLE(cellpadding="5", #cellspacing="5", border="5", 
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Chip annotations for:"), 
                               TD(x@chip)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Chip data package:"), 
                               TD(chippkg)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "GO data package:"), 
                               TD(GOpkg)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Root node chosen:"), 
                               TD(x@root, "(", getGOterm(x@root), ")")),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                  "Number of probesets<BR>on the chip:"), 
                               TD(length(ls(x@probes)))),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                  "Probesets annotated<BR>below ", x@root, ":"), 
                               TD(nprobes)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                  "Number of nodes<BR>with direct annotations:"), 
                               TD(length(x@leafs))),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Total number of nodes:"), 
                               TD(length(x@nodes))))))

    if (nonodes) return(invisible(NULL))
    catHTML(HR(), NAME("nodesinfo"), H2("StAM-Net details about GO-nodes"))
    l <- length(x@nodes)
    .stam.allaffyids <<- ls(getprobes(x@chip))    
    .stam.allnodes <<- x@nodes
    .stam.GOids <<- new.env(hash=TRUE)
    for (i in 1:length(.stam.allnodes)) 
      assign(.stam.allnodes[[i]]@ID, i, .stam.GOids)
    for (i in l:1) {
      if (((l-i) %% 100) == 0) cat("written HTML for", l-i, "of", l, "nodes\n")
      writeHTML(x@nodes[[i]], sepfiles=FALSE)
    }
    rm(.stam.allaffyids, .stam.allnodes, .stam.GOids, envir=.GlobalEnv)
  }
)


################################################################################
#
#   Plots and Graphs
#



#
#   End of File
#
################################################################################

==> stam_1.6.0.tar.gz: stam/R/03_net.R <==
################################################################################
#
#   03_stamNet.R
#
#   Structural information on a whole graph given a specific
#   affymetrix chip and a root node. Holds an array of GO-nodes
#   in postorder, introducing additional nodes for gene annotations.
#   Nodes holding only one child after pruning all children
#   without gene annotations are replaced by that child.
#
#   The implementation relies on Bioconductor data packages to
#   extract chip-specific data as well as the GO structure.
#
#   written by Claudio Lottaz (COMPDIAG-MPI-MG), in August 2004
#
################################################################################
#
#   Class Definition and Constructor
#

setClass("stamNet", 
         representation(chip     = "character",     # chip to produce data
                        root     = "character",     # where to start the analyses
                        chippkg  = "character",     # package with chip-related data
                        GOpkg    = "character",     # package with GO-related data
                        nodes    = "list",          # stamINodes and stamLeafs
                        leafs    = "numeric",       # indeces to leafs
                        inodes   = "numeric",       # indeces to inner nodes
                        probes   = "environment"))  # holds indeces of affyids


setMethod("initialize", "stamNet", 
  function(.Object, chip="hgu95av2", root="GO:0008150", probes=character(0)) {

    .Object@chip <- chip
    .Object@root <- root

    # find chip-specific data
    if (all(.packages(all=TRUE) != chip)) {
      dataRepository <- biocReposList()["aData"]
      install.packages(chip, lib=.libPaths()[1], repos=dataRepository)
      if (all(.packages(all=TRUE) != chip)) 
        stop("No data for chip ", chip, " has been found")
    }
    chipdsc <- packageDescription(chip)
    GOdsc   <- packageDescription("GO")
    .Object@chippkg <- paste("Chip data package:\n  version", chipdsc$Version, 
                             "\n  ", chipdsc$Packaged)
    .Object@GOpkg   <- paste("GO package:\n  version",   GOdsc$Version,   
                             "\n  ", GOdsc$Packaged)
    require(package=chip, character.only=TRUE)

    # check probes
    chipprobes <- getprobes(chip)
    if (length(probes) > 0) { 
      .Object@probes <- new.env(hash=TRUE)
      for (i in 1:length(probes)) {
        if (!exists(probes[i], chipprobes)) 
          stop("Rownames of expression matrix do not match '", chip, "'")
        assign(probes[i], i, .Object@probes)
      }
    } else { .Object@probes <- chipprobes }

    # construct the graph
    chipenv <- get(paste(chip, "GO2PROBE", sep=""), paste("package:", chip, sep=""))
    .stam.nodes     <<- list()
    .stam.nnodes    <<- 0
    .stam.leafs     <<- logical(0)    
    .stam.GOidx     <<- new.env(hash=TRUE)
    environment(stam.addNode) <- environment()
    postorder.traverse(root, stam.addNode)
    names(.stam.nodes) <- sapply(.stam.nodes, function(x) x@ID)
    .Object@nodes <- .stam.nodes
    .Object@leafs  <- which(.stam.leafs)
    .Object@inodes <- which(!.stam.leafs)
    rm(.stam.leafs, .stam.nodes, .stam.nnodes, .stam.GOidx, envir=.GlobalEnv)
    return(.Object)
  }
)


################################################################################
#
#   Postorder Traversal
#


postorder.traverse <- function(ID, action, env) { 
  postorder.traverse.rec <- function(ID, action, env) {
    if (exists(ID, env)) return(FALSE)
    assign(ID, 1, env)
    nvisited <- get("nvisited", env)
    assign("nvisited", nvisited + 1, env)
    if ((nvisited %% 500) == 0)
      cat("registered",.stam.nnodes,"of",nvisited,"visited nodes\n")

    for (c in getGOchildren(ID)) postorder.traverse.rec(c, action, env)
    return(action(ID))
  }

  env <- new.env(hash=TRUE)
  assign("nvisited", 0, env)
  assign("visited", character(0), env)
  postorder.traverse.rec(ID, action, env)
  cat("registered", .stam.nnodes, "of", get("nvisited", env), "visited nodes\n")
  rm(env)
}

stam.addNode <- function(ID) {
  children <- stam.getidx(getGOchildren(ID), .stam.GOidx)
  nchildren <- length(children)
  if (exists(ID, chipenv)) {  # current node has probesets annotated
    l <- new("stamLeaf",  ID=ID, chip=chip, .Object@probes)
    if (nchildren > 0) { 
      chnames <- c(unlist(lapply(children, function(c) .stam.nodes[[c]]@ID)), ID)
      children <- c(children, .stam.nnodes+1)
      names(children) <- chnames
      n <- new("stamINode", ID=ID, children=children)
      .stam.nnodes <<- .stam.nnodes + 2
      l@supNode <- TRUE
      .stam.nodes[[.stam.nnodes-1]] <<- l; .stam.leafs[.stam.nnodes-1] <<- TRUE
      .stam.nodes[[.stam.nnodes]]   <<- n; .stam.leafs[.stam.nnodes]   <<- FALSE
      assign(ID, .stam.nnodes, .stam.GOidx)
    } else {
      .stam.nnodes <<- .stam.nnodes + 1
      .stam.nodes[[.stam.nnodes]] <<- l; .stam.leafs[.stam.nnodes] <<- TRUE
      assign(ID, .stam.nnodes, .stam.GOidx)
    }
  } else { # current node has no probesets annotated
    if (nchildren > 1) { 
      names(children) <- unlist(lapply(children, function(c) .stam.nodes[[c]]@ID))
      n <- new("stamINode", ID=ID, children=children)
      .stam.nnodes <<- .stam.nnodes + 1
      .stam.nodes[[.stam.nnodes]] <<- n; .stam.leafs[.stam.nnodes] <<- FALSE
      assign(ID, .stam.nnodes, .stam.GOidx)
    } else {
      if (nchildren == 1) { # replace current node by its only childee
        child <- children[1]
	        reps <- length(.stam.nodes[[child]]@replacedParents)
        .stam.nodes[[child]]@replacedParents[reps+1] <<- ID
        assign(ID, child, .stam.GOidx)
      } else {
        return(FALSE)
      }
    }
  }
  return(TRUE)
}

################################################################################
#
#   Reports
#

setMethod("print", "stamNet", 
  function(x, ...) {
    allGOenv <- get(paste(x@chip, "GO2ALLPROBES", sep=""),
                    paste("package:", x@chip, sep=""))
    nprobes <- length(unique(get(x@root, allGOenv)))
    allprobes <- length(ls(x@probes))

    cat("stamNet on chip ", x@chip, "\n", x@chippkg, "\n",
        "Gene Ontology ", x@GOpkg, ")\n",
        "root is set to \n", x@root, " (", getGOterm(x@root), ")\n", 
        "holds ", length(x@nodes), " nodes, ", length(x@leafs), 
        " of which are leafs and ", length(x@inodes), " are inner nodes\n", 
        nprobes, " of ", allprobes, " probesets have annotations", 
        " (", round(nprobes/allprobes*100, 1), "%)\n", sep="")

  }
)

setMethod("writeHTML", "stamNet", 
  function(x, nonodes=FALSE, ...) {
    allGOenv <- get(paste(x@chip, "GO2ALLPROBES", sep=""),
                    paste("package:", x@chip, sep=""))

    nprobes <- length(unique(get(x@root, allGOenv)))
    chippkg <- sub(" / ", "<BR>", x@chippkg)
    GOpkg <- sub(" / ", "<BR>", x@GOpkg)

    catHTML(HR(), NAME("netinfo"), H2("Annotation Information"), 
            P(TABLE(cellpadding="5", #cellspacing="5", border="5", 
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Chip annotations for:"), 
                               TD(x@chip)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Chip data package:"), 
                               TD(chippkg)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "GO data package:"), 
                               TD(GOpkg)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Root node chosen:"), 
                               TD(x@root, "(", getGOterm(x@root), ")")),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                  "Number of probesets<BR>on the chip:"), 
                               TD(length(ls(x@probes)))),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                  "Probesets annotated<BR>below ", x@root, ":"), 
                               TD(nprobes)),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", 
                                  "Number of nodes<BR>with direct annotations:"), 
                               TD(length(x@leafs))),
              TR(valign="top", TH(align="right", bgcolor="lightgreen", "Total number of nodes:"), 
                               TD(length(x@nodes))))))

    if (nonodes) return(invisible(NULL))
    catHTML(HR(), NAME("nodesinfo"), H2("StAM-Net details about GO-nodes"))
    l <- length(x@nodes)
    .stam.allaffyids <<- ls(getprobes(x@chip))    
    .stam.allnodes <<- x@nodes
    .stam.GOids <<- new.env(hash=TRUE)
    for (i in 1:length(.stam.allnodes)) 
      assign(.stam.allnodes[[i]]@ID, i, .stam.GOids)
    for (i in l:1) {
      if (((l-i) %% 100) == 0) cat("written HTML for", l-i, "of", l, "nodes\n")
      writeHTML(x@nodes[[i]], sepfiles=FALSE)
    }
    rm(.stam.allaffyids, .stam.allnodes, .stam.GOids, envir=.GlobalEnv)
  }
)


################################################################################
#
#   Plots and Graphs
#



#
#   End of File
#
################################################################################
